﻿// DecompositionComposition (c) 2009 by Charles Petzold
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace DecompositionComposition
{
    public partial class MainPage : UserControl
    {
        List<PointPair> txt1Lines = new List<PointPair>();
        List<PointPair> txt2Lines = new List<PointPair>();
        static readonly Duration fadeDuration = new Duration(TimeSpan.FromSeconds(3));
        static readonly Duration lineDuration = new Duration(TimeSpan.FromSeconds(10));

        public MainPage()
        {
            InitializeComponent();

            // Get PathGeometry and break into lines and sort
            BreakIntoLines(txt1Lines, txt1Geometry);
            txt1Lines.Sort(ComparePointPairs);

            // Get second PathGeometry and break into lines and sort
            BreakIntoLines(txt2Lines, txt2Geometry);
            txt2Lines.Sort(ComparePointPairs);

            // Equalize the number of lines in each collection
            int numLines = Math.Max(txt1Lines.Count, txt2Lines.Count);
            List<PointPair> txtLinesInsufficient = txt1Lines.Count < txt2Lines.Count ? txt1Lines : txt2Lines;

            while (txtLinesInsufficient.Count < numLines)
            {
                PointPair pointPair = txtLinesInsufficient[0];
                txtLinesInsufficient.RemoveAt(0);
                Point middlePoint = new Point((pointPair.Point1.X + pointPair.Point2.X) / 2,
                                              (pointPair.Point1.Y + pointPair.Point2.Y) / 2);

                PointPair pointPair1 = new PointPair()
                {
                    Point1 = pointPair.Point1,
                    Point2 = middlePoint
                };
                txtLinesInsufficient.Add(pointPair1);

                PointPair pointPair2 = new PointPair()
                {
                    Point1 = middlePoint,
                    Point2 = pointPair.Point2
                };
                txtLinesInsufficient.Add(pointPair2);

                txtLinesInsufficient.Sort(ComparePointPairs);
            }

            // Reverse one of them so short <--> long lines
            txt2Lines.Reverse();

            // Main storyboard
            Storyboard storyboardMain = new Storyboard()
            {
                BeginTime = TimeSpan.FromSeconds(3),
                AutoReverse = true,
                RepeatBehavior = RepeatBehavior.Forever
            };

            // First storyboard: Fade out txt1
            Storyboard storyboard1 = new Storyboard();

            DoubleAnimation animaFade1 = new DoubleAnimation()
            {
                From = 1,
                To = 0,
                Duration = fadeDuration
            };
            Storyboard.SetTarget(animaFade1, txt1Path);
            Storyboard.SetTargetProperty(animaFade1, new PropertyPath(UIElement.OpacityProperty));
            storyboard1.Children.Add(animaFade1);
            storyboardMain.Children.Add(storyboard1);

            // Second storyboard: Move all the lines we're creating here
            Brush blackBrush = new SolidColorBrush(Colors.Black);
            Storyboard storyboard2 = new Storyboard()
            {
                BeginTime = fadeDuration.TimeSpan
            };

            for (int index = 0; index < Math.Min(txt1Lines.Count, txt2Lines.Count); index++)
            {
                LineGeometry lineGeometry = new LineGeometry()
                {
                    StartPoint = txt1Lines[index].Point1,
                    EndPoint = txt1Lines[index].Point2
                };

                Path path = new Path()
                {
                    Stroke = blackBrush,
                    StrokeThickness = 2,
                    Data = lineGeometry
                };
                textContainer.Children.Add(path);

                PointAnimation anima = new PointAnimation()
                {
                    From = txt1Lines[index].Point1,
                    To = txt2Lines[index].Point1,
                    Duration = lineDuration,
                    EasingFunction = new BackEase()
                    {
                        EasingMode = EasingMode.EaseInOut
                    }
                };
                Storyboard.SetTarget(anima, lineGeometry);
                Storyboard.SetTargetProperty(anima, new PropertyPath(LineGeometry.StartPointProperty));
                storyboard2.Children.Add(anima);

                anima = new PointAnimation()
                {
                    From = txt1Lines[index].Point2,
                    To = txt2Lines[index].Point2,
                    Duration = lineDuration,
                    EasingFunction = new BackEase()
                    {
                        EasingMode = EasingMode.EaseInOut
                    }
                };
                Storyboard.SetTarget(anima, lineGeometry);
                Storyboard.SetTargetProperty(anima, new PropertyPath(LineGeometry.EndPointProperty));
                storyboard2.Children.Add(anima);
            }
            storyboardMain.Children.Add(storyboard2);

            // Third storyboard: Fade in second text
            Storyboard storyboard3 = new Storyboard()
            {
                BeginTime = fadeDuration.TimeSpan + lineDuration.TimeSpan
            };

            DoubleAnimation animaFade2 = new DoubleAnimation()
            {
                From = 0,
                To = 1,
                Duration = fadeDuration
            };
            Storyboard.SetTarget(animaFade2, txt2Path);
            Storyboard.SetTargetProperty(animaFade2, new PropertyPath(UIElement.OpacityProperty));
            storyboard3.Children.Add(animaFade2);
            storyboardMain.Children.Add(storyboard3);

            storyboardMain.Begin();
        }

        void BreakIntoLines(List<PointPair> txtLines, PathGeometry pathGeometry)
        {
            foreach (PathFigure pathFigure in pathGeometry.Figures)
            {
                Point point1 = pathFigure.StartPoint;

                if (pathFigure.Segments.Count != 1)
                    throw new InvalidOperationException("Expecting only one PathSegment");

                PointCollection points = (pathFigure.Segments[0] as PolyLineSegment).Points;

                for (int i = 0; i < points.Count; i++)
                {
                    PointPair pointPair = new PointPair()
                    {
                        Point1 = point1,
                        Point2 = points[i]
                    };
                    txtLines.Add(pointPair);

                    point1 = points[i];
                }

                // Close the figure if necessary
                if (point1 != pathFigure.StartPoint)
                {
                    PointPair pointPair = new PointPair()
                    {
                        Point1 = point1,
                        Point2 = pathFigure.StartPoint
                    };
                    txtLines.Add(pointPair);
                }
            }
        }

        // Sorts longest to shortest
        int ComparePointPairs(PointPair pointPair1, PointPair pointPair2)
        {
            return pointPair2.LengthSquared.CompareTo(pointPair1.LengthSquared);
        }
    }
    public struct PointPair
    {
        public Point Point1 { set; get; }
        public Point Point2 { get; set; }
        public double LengthSquared
        {
            get
            {
                return Math.Pow(Point1.X - Point2.X, 2) + Math.Pow(Point1.Y - Point2.Y, 2);
            }
        }
    }
}
